
import java.util.Random;
import java.util.UUID;
import org.diligentproject.searchservice.searchlibrary.resultset.elements.ResultElementBase;
import org.diligentproject.searchservice.searchlibrary.resultset.elements.ResultElementGeneric;
import org.diligentproject.searchservice.searchlibrary.rsclient.elements.RSLocator;
import org.diligentproject.searchservice.searchlibrary.rsclient.elements.RSResourceLocalType;
import org.diligentproject.searchservice.searchlibrary.rsreader.RSXMLIterator;
import org.diligentproject.searchservice.searchlibrary.rsreader.RSXMLReader;
import org.diligentproject.searchservice.searchlibrary.rswriter.RSXMLWriter;

public class RSExamples {
	private static Random gen=new Random();
	
	public static void main(String []args) throws Exception{
		System.out.println("----------------creating the RS----------------");
		RSXMLWriter writer=createRSWriter();
		System.out.println("----------------reading the RS----------------");
		readRS(writer.getRSLocator(new RSResourceLocalType()));
		System.out.println("----------------iterating over the RS----------------");
		iterateRS(writer.getRSLocator(new RSResourceLocalType()));
		System.out.println("----------------partialy localizing the RS----------------");
		RSLocator local=localizePart(writer.getRSLocator(new RSResourceLocalType()),10);
		System.out.println("----------------iterating over the RS----------------");
		iterateRS(local);
	}
	
	/*A method that creates a new RSXMLWriter and populates it with some results*/
	public static RSXMLWriter createRSWriter() throws Exception{
		/*Create a new writer with the default parametrizable values*/ 
		RSXMLWriter writer=RSXMLWriter.getRSXMLWriter(); 
		/*Create a new writer which should have as a condition for paging inserted results either having 
		 30 records per page or having a total size surpasing 1024 bytes */ 
		//writer=RSXMLWriter.getRSXMLWriter(30,1024); 
		/*Create a new writer and set a lifetime property of 1000 millisecs. Also set that the paging 
		 condition will be having 10 records per page or a total size surpasing teh default value*/ 
		//writer=RSXMLWriter.getRSXMLWriter(new PropertyElementBase[]{new PropertyElementLifeSpanGC(1000)}); 
		//writer.setRecsPerPart(10); 
		/*Add a new result with the given id, collection, rank and payload*/
		for(int i=0;i<90;i+=1){
			writer.addResults(new ResultElementGeneric(UUID.randomUUID().toString(),"collection1",""+gen.nextFloat(),"<Record><a><b>"+gen.nextInt()+"</b></a></Record>"));
		}
		/*Give back the writer*/ 
		/*One should ALWAYS close the RS he is creating in order to make the full payload available to readers*/
		writer.close(); 
		return writer; 
		
	}
	
	/*This method instantiates a new RSXMLReader to point to the RS identified by the provided RSLocator and iterates over each page retrieving the contents*/
	public static void readRS(RSLocator locator) throws Exception {
		
		RSXMLReader reader=RSXMLReader.getRSXMLReader(locator); 
		while(!reader.isLast()){
			/*Retrieve the records in the current page as instances of the provided class that extends the ResultElementBase*/ 
			ResultElementBase []res=reader.getResults(ResultElementGeneric.class);
			for(int i=0;i<res.length;i+=1){
				System.out.println(((ResultElementGeneric)res[i]).getPayload());
			}
			reader.getNextPart(); 
		} 
		reader.getFirstPart(); 
		do{
			System.out.println(reader.getNumberOfResults()); 
		}while(reader.getNextPart()); 
		
	} 
	
	/*This method instantiates a new RSXMLReader to point to the RS identified by the provided RSLocator and iterates over each record retrieving the contents*/
	public static void iterateRS(RSLocator locator) throws Exception {
		
		RSXMLIterator iter=RSXMLReader.getRSXMLReader(locator).getRSIterator(); 
		while(iter.hasNext()){
			ResultElementGeneric elem=(ResultElementGeneric)iter.next(ResultElementGeneric.class); 
			/*Retrieve the DocID of the retrieved record*/ 
			if(elem!=null) System.out.println(elem.getRecordAttributes(ResultElementGeneric.RECORD_ID_NAME)[0].getAttrName()+":"+elem.getRecordAttributes(ResultElementGeneric.RECORD_ID_NAME)[0].getAttrValue());
		} 
		
	} 
	
	/*This method localizes a ResultSet keeping the top count records of the identified throught the RSLocator RS. The localization is in terms of the node that hosts the RS content and not the Resource type. This can be further defined by the RSResourceType*/
	public static RSLocator localizePart(RSLocator locator,int count) throws Exception {
		RSXMLReader reader=RSXMLReader.getRSXMLReader(locator);
		RSXMLReader readerLocalTop=reader.makeLocal(new RSResourceLocalType(),count); 
		readerLocalTop.isLocal(); 
		return readerLocalTop.getRSLocator();
	} 
}
